#include "stdafx.h"
#include "func.h"
#include "tex.h"
#include "TeXProject.h"
#include "Analyze.h"
#include "Setting.h"
#include "TeXSet.h"
#include "MakeIndexSet.h"
#include "BibTeXSet.h"
#include "DVI2PDFSet.h"
#include "DVI2PSSet.h"
#include "DVIOUTSet.h"
#include "MainWnd.h"
#include "DdeClient.h"
#include <regex>
#include <ablib/IsKanji.h>
#include <filesystem>

extern Setting g_setting;

TeX::ToDO TeX::GetToDO(const ablib::string &file) {
	ablib::string ext = std::tr2::sys::path(file).extension().string();
	ext.tolower();
	if (ext == _T(".tex"))return tex2dvi;
	else if (ext == _T(".dvi"))return dvi2pdf;
	else if (ext == _T(".ps"))return ps2pdf;
	else {
		DEBUGSTRING(_T("o^ĂȂgqD"));
		return none;
	}
}

bool TeX::DO(ToDO todo){
	switch(todo){
	case tex2dvi:	return TeX2DVI();
	case dvi2ps:	return DVI2PS();
	case dvi2pdf:	return DVI2PDF();
	case ps2pdf:	return PS2PDF();
	default:		return false;
	}
}

void TeX::GetDocumentInfo() {
	std::tr2::sys::path path(m_filename.c_str());
	m_type = path.extension().string();
	m_dir = path.parent_path().string();
	m_base = path.stem().string();
	m_file = m_dir + _T("\\") + m_base;

	if (m_type == _T(".tex")) {
		TeXProject tp;
		switch (g_setting.TeX_WhichParent()) {
		case TeXSet::now:
			if (::CheckSameFile(tp.FindParentFile(m_filename.c_str()), m_filename.c_str())) {
				m_documentoption = tp.GetDocumentOption();
				m_documentclass = tp.GetDocumentClass();
			} else {
				m_documentoption = _T("");
				m_documentclass = _T("");
			}
			m_parent = m_filename;
			break;

		case TeXSet::sitei:
			m_parent = g_setting.TeX_Parent();
			m_documentoption = _T("");
			m_documentclass = _T("");
			break;

		case TeXSet::guess:
		default:
			m_parent = tp.FindParentFile(m_filename.c_str());
			m_documentoption = tp.GetDocumentOption();
			m_documentclass = tp.GetDocumentClass();
			break;

		}
		if (m_parent != _T("")) {
			path = std::tr2::sys::path(m_parent.c_str());
			m_parenttype = path.extension().string();
			m_parentdir = path.parent_path().string();
			m_parentbase = path.stem().string();
			m_parent = m_parentdir + _T("\\") + m_parentbase;
		}
	} else {
		m_parent = m_file;
		m_parenttype = m_type;
		m_parentdir = m_dir;
		m_parentbase = m_base;
		m_documentoption = _T("");
		m_documentclass = _T("");
	}
	return;
}

bool TeX::TeX2DVI() {
	if (m_filename == _T("")) {
		m_mainwnd->GetWindow()->MsgBox(_T("t@Cǂݍ܂Ă܂D"), _T("TeXShell2"));
		return false;
	}
	ablib::string file = m_parent + m_parenttype;
	DEBUGSTRING(_T("%s"), file.c_str());
	if (m_parent == _T("")) {
		m_mainwnd->GetWindow()->MsgBox(_T("et@C̎擾ɎsD"), _T("TeXShell2"));
		return false;
	}
	if (::GetFileAttributes(file.c_str()) == -1) {
		m_mainwnd->GetWindow()->MsgBox(_T("t@C݂܂D"), _T("TeXShell2"));
		return false;
	}
	if (g_setting.TeX2DVI_IsAnalyze()) {
		Analyze ana;
		ablib::string cmd = ana.Start(file.c_str());
		if (cmd == _T(""))cmd = _T("latex\n") + m_parent;
		int max = g_setting.TeX2DVI_MaxCompile();
		for (int i = 0; i < max; ++i) {
			if (cmd == _T(""))break;
			DEBUGSTRING(_T("TeX::TeX2DVI : ANALYZE_***́u%svԂ܂D"), cmd.c_str());
			ablib::string::size_type p = cmd.find(_T("\n"));
			ablib::string anac = cmd.substr(0, p); anac.tolower();
			ablib::string f = cmd.substr(p + 1);
			ablib::string c;
			if (anac == _T("latex")) {
				if (g_setting.TeX2DVI_IsUsingFormat()) {
					c = g_setting.TeX2DVI_FormatFile();
					c.erase(c.length() - 4);
				} else c = g_setting.TeX2DVI_Program();
				if (c == _T("")) {
					m_mainwnd->GetWindow()->MsgBox(_T("TeXvO̐ݒ肪Ă܂D"), _T("TeXShell2"));
					ana.Finish();
					return false;
				}
				if (g_setting.TeX2DVI_Option() != _T(""))c += _T(" ") + g_setting.TeX2DVI_Option();
				std::tr2::sys::path path(f.c_str());
				ablib::string d = path.parent_path().string();
				f = path.filename().string();
				c += _T(" ") + f;
				if (!m_mainwnd->Exec(c, d) || m_mainwnd->GetReturnCode() != 0) {
					m_mainwnd->GetWindow()->MsgBox(c + _T("̎sɎsD"), _T("TeXShell2"));
					ana.Finish();
					return false;
				}
			} else if (anac == _T("makeindex")) {
				if (!MakeIndex(f)) {
					ana.Finish();
					return false;
				} else if (m_mainwnd->GetReturnCode() != 0) {
					m_mainwnd->GetWindow()->SetTitle(_T("MakeIndexsƂCG[R[h") + ablib::lexical_cast<ablib::string>(m_mainwnd->GetReturnCode()) + _T("Ԃ܂Ĉ܂܎s𑱂܂D"));
				}
			} else if (anac == _T("bibtex")) {
				if (!BibTeX(f)) {
					ana.Finish();
					return false;
				} else if (m_mainwnd->GetReturnCode() != 0) {
					m_mainwnd->GetWindow()->SetTitle(_T("BibTeXsƂCG[R[h") + ablib::lexical_cast<ablib::string>(m_mainwnd->GetReturnCode()) + _T("Ԃ܂Ĉ܂܎s𑱂܂D"));
				}
			} else ASSERT(false);
			cmd = ana.Next();
		}
		ana.Finish();
		return true;
	} else {
		ablib::string c;
		if (g_setting.TeX2DVI_IsUsingFormat()) {
			c = g_setting.TeX2DVI_FormatFile();
			c.erase(c.length() - 4);
		} else c = g_setting.TeX2DVI_Program();
		if (c == _T("")) {
			m_mainwnd->GetWindow()->MsgBox(_T("TeXvO̐ݒ肪Ă܂D"), _T("TeXShell2"));
			return false;
		}
		if (g_setting.TeX2DVI_Option() != _T(""))c += _T(" ") + g_setting.TeX2DVI_Option();
		c += _T(" ") + m_parentbase;
		if (!m_mainwnd->Exec(c, m_parentdir)) {
			m_mainwnd->GetWindow()->MsgBox((c + _T("̎sɎsD")).c_str(), _T("TeXShell2"));
			return false;
		} else if (m_mainwnd->GetReturnCode() != 0) {
			m_mainwnd->GetWindow()->MsgBox(_T("RpC̓G[Ԃ܂D"), _T("TeXShell2"));
			return false;
		} else return true;
	}
}

bool TeX::MakeIndex(const ablib::string &file) {
	std::tr2::sys::path path(file.c_str());
	ablib::string f = (path.parent_path() / path.stem()).string();
	ablib::string d = path.parent_path().string();
	ablib::string c;
	switch (g_setting.MakeIndex_WhichProgram()) {
	case MakeIndexSet::makeindex:	c = _T("makeindex"); break;
	case MakeIndexSet::other:		c = g_setting.MakeIndex_Program(); break;
	default:
	case MakeIndexSet::mendex:		c = _T("mendex"); break;
	}
	if (c == _T("")) {
		m_mainwnd->GetWindow()->MsgBox(_T("MakeIndex̐ݒ肪ȂĂ܂D"), _T("TeXShell2"));
		return false;
	}
	if (g_setting.MakeIndex_Option() != _T(""))c += _T(" ") + g_setting.MakeIndex_Option();
	c += _T(" ") + f;
	if (!m_mainwnd->Exec(c, d)) {
		m_mainwnd->GetWindow()->MsgBox((c + _T("̎sɎsD")).c_str(), _T("TeXShell2"));
		return false;
	} else return true;
}

bool TeX::BibTeX(const ablib::string &file) {
	std::tr2::sys::path path(file.c_str());
	ablib::string f = (path.parent_path() / path.stem()).string();
	ablib::string d = path.parent_path().string();
	ablib::string c;
	switch (g_setting.BibTeX_WhichProgram()) {
	case BibTeXSet::bibtex:		c = _T("bibtex"); break;
	case BibTeXSet::other:		c = g_setting.BibTeX_Program(); break;
	default:
	case BibTeXSet::jbibtex:	c = Getjpbibtex(); break;
	}
	if (c == _T("")) {
		m_mainwnd->GetWindow()->MsgBox(_T("BibTeX̐ݒ肪ȂĂ܂D"), _T("TeXShell2"));
		return false;
	}
	if (g_setting.BibTeX_Option() != _T(""))c += _T(" ") + g_setting.BibTeX_Option();
	c += _T(" ") + f;
	if (!m_mainwnd->Exec(c, d)) {
		m_mainwnd->GetWindow()->MsgBox((c + _T("̎sɎsD")).c_str(), _T("TeXShell2"));
		return false;
	} else return true;
}

const TCHAR *TeX::Getjpbibtex() {
	if (::lstrcmp(::Which(_T("pbibtex")), _T("")) != 0)return _T("pbibtex");
	else return _T("jbibtex");
}

bool TeX::DVI2PDF(){
	if(m_filename == _T("")){
		m_mainwnd->GetWindow()->MsgBox(_T("t@Cǂݍ܂Ă܂D"),_T("TeXShell2"));
		return false;
	}
	if (m_parent == _T("")) {
		m_mainwnd->GetWindow()->MsgBox(_T("et@C̎擾ɎsD"), _T("TeXShell2"));
		return false;
	}
	ablib::string c;
	switch(g_setting.DVI2PDF_WhichProgram()){
	case DVI2PDFSet::other:
		c = g_setting.DVI2PDF_Program();
		break;
	default:
	case DVI2PDFSet::dvipdfmx:
		c = _T("dvipdfmx");
		break;
	}
	if(c == _T("")){
		m_mainwnd->GetWindow()->MsgBox(_T("DVIPDFϊ̐ݒ肪ȂĂ܂D"),_T("TeXShell2"));
		return false;
	}
	//if(g_setting.DVI2PDF_PaperSizeFromList() != DVI2PDFSet::other){
	if (g_setting.DVI2PDF_PaperSizeFromList() != _T("other")) {
		switch(g_setting.DVI2PDF_WhichPaperSize()){
		case DVI2PDFSet::guess:
		{
			struct PSizeOpt{
				TCHAR *psize;
				TCHAR *opt;
			};
			PSizeOpt psop[] = {
				{_T("a3")		,_T("(A3J?|a3paper)")	},
				{_T("a4")		,_T("(A4J?|a3paper)")	},
				{_T("a5")		,_T("(A5J?|a3paper)")	},
				{_T("a6")		,_T("(A6J?|a3paper)")	},
				{_T("b3")		,_T("(B3J?|b3paper)")	},
				{_T("b4")		,_T("(B4J?|b3paper)")	},
				{_T("b5")		,_T("(B5J?|b3paper)")	},
				{_T("b6")		,_T("(B6J?|b3paper)")	},
				{_T("letter")	,_T("letterpaper")		},
				{_T("legal")	,_T("legalpaper")		},
				{NULL			,NULL					}
			};
#ifdef UNICODE
			std::wregex reg;
#else
			std::regex reg;
#endif
			for(PSizeOpt *i = psop ; i->psize ; ++i){
				reg.assign(i->opt,std::regex::icase);
				if(std::regex_match(m_documentoption,reg)){
					c += ablib::string(_T(" -p ")) + i->psize;
					break;
				}
			}
			break;
		}
		case DVI2PDFSet::list:
			if(g_setting.DVI2PDF_PaperSizeFromList() != _T("")){
				c += _T(" ") + g_setting.DVI2PDF_PaperSizeFromList();
			}
			break;
		case DVI2PDFSet::number:
		{
			ablib::string x = g_setting.DVI2PDF_PaperSizeX();
			ablib::string y = g_setting.DVI2PDF_PaperSizeY();
			if(x != _T("") && y != _T("")){
				c += _T(" ") + x + _T(",") + y;
			}
			break;
		}
		default:
			break;
		}
	}
	if(g_setting.DVI2PDF_Option() != _T("")){
		c += _T(" ") + g_setting.DVI2PDF_Option();
	}
	c += _T(" ") + m_parentbase + _T(".dvi");
	if(!m_mainwnd->Exec(c,m_parentdir)){
		m_mainwnd->GetWindow()->MsgBox(c + _T("̎sɎsD"),_T("TeXShell2"));
		return false;
	}else if(m_mainwnd->GetReturnCode() != 0){
		m_mainwnd->GetWindow()->MsgBox(
			_T("G[R[h") + ablib::lexical_cast<ablib::string>(m_mainwnd->GetReturnCode()) + 
			_T("AĂ܂D"),_T("TeXShell2"));
		return false;
	}else return true;
}

bool TeX::DVI2PS() {
	if (m_filename == _T("")) {
		m_mainwnd->GetWindow()->MsgBox(_T("t@Cǂݍ܂Ă܂D"), _T("TeXShell2"));
		return false;
	}
	if (m_parent == _T("")) {
		m_mainwnd->GetWindow()->MsgBox(_T("et@C̎擾ɎsD"), _T("TeXShell2"));
		return false;
	}
	ablib::string c;
	switch (g_setting.DVI2PS_WhichProgram()) {
	case DVI2PSSet::dvi2ps:
		c = _T("dvi2ps");
		break;
	case DVI2PSSet::other:
		c = g_setting.DVI2PS_Program();
		break;
	case DVI2PSSet::dvipsk:
	default:
		//		c = _T("dvipsk");
		c = _T("dvips");
		break;
	}
	if (c == _T("")) {
		m_mainwnd->GetWindow()->MsgBox(_T("DVIPSϊ̐ݒ肪ȂĂ܂D"), _T("TeXShell2"));
		return false;
	}
	if (g_setting.DVI2PS_Option() != _T(""))c += _T(" ") + g_setting.DVI2PS_Option();
	c += _T(" ") + m_parentbase + _T(".dvi");
	if (!m_mainwnd->Exec(c, m_parentdir)) {
		m_mainwnd->GetWindow()->MsgBox(c + _T("̎sɎs܂D"), _T("TeXShell2"));
		return false;
	} else if (m_mainwnd->GetReturnCode() != 0) {
		m_mainwnd->GetWindow()->MsgBox(
			_T("G[R[h") + ablib::lexical_cast<ablib::string>(m_mainwnd->GetReturnCode()) + _T("AĂ܂D"),
			_T("TeXShell2"));
		return false;
	} else return true;
}

bool TeX::PS2PDF() {
	if (m_filename == _T("")) {
		m_mainwnd->GetWindow()->MsgBox(_T("t@Cǂݍ܂Ă܂D"), _T("TeXShell2"));
		return false;
	}
	if (m_parent == _T("")) {
		m_mainwnd->GetWindow()->MsgBox(_T("et@C̎擾ɎsD"), _T("TeXShell2"));
		return false;
	}
	ablib::string c;
	if (g_setting.PS2PDF_IsGhostScript())c = _T("ps2pdf.bat");
	else c = g_setting.PS2PDF_Program();
	if (c == _T("")) {
		m_mainwnd->GetWindow()->MsgBox(_T("PSPDFϊ̐ݒ肪ȂĂ܂D"), _T("TeXShell2"));
		return false;
	}
	if (g_setting.PS2PDF_Option() != _T(""))c += _T(" ") + g_setting.PS2PDF_Option();
	c += _T(" ") + m_parentbase + _T(".ps ") + m_parentbase + _T(".pdf");
	if (!m_mainwnd->Exec(c, m_parentdir)) {
		m_mainwnd->GetWindow()->MsgBox(c + _T("̎sɎs܂D"), _T("TeXShell2"));
		return false;
	} else if (m_mainwnd->GetReturnCode() != 0) {
		m_mainwnd->GetWindow()->MsgBox(
			_T("G[R[h") + ablib::lexical_cast<ablib::string>(m_mainwnd->GetReturnCode()) + _T("Ԃ܂D"),
			_T("TeXShell2"));
		return false;
	} else return true;
}

bool TeX::DVIPreview() {
	if (m_filename == _T("")) {
		m_mainwnd->GetWindow()->MsgBox(_T("t@Cǂݍ܂Ă܂D"), _T("TeXShell2"));
		return false;
	}
	if (m_parent == _T("")) {
		m_mainwnd->GetWindow()->MsgBox(_T("et@C̎擾ɎsD"), _T("TeXShell2"));
		return false;
	}
	if (!g_setting.DVIOUT_IsUseDVIOUT()) {
		ablib::string prog = g_setting.DVIOUT_Program();
		ablib::string arg = g_setting.DVIOUT_Arg();
		arg = TransStr(arg);
		ablib::string cmd = _T("\"") + prog + _T("\" ") + arg;
		if (!Exec(cmd)) {
			m_mainwnd->GetWindow()->MsgBox(cmd + _T("̎sɎsD"), _T("TeXShell2"));
			return false;
		} else return true;
	}
	ablib::string papersize;
	if (g_setting.DVIOUT_IsTellPaperSize()) {
		switch (g_setting.DVIOUT_WhichPaperSize()) {
		case DVIOUTSet::guess:
		{
			struct PSizeOpt {
				TCHAR *psize;
				TCHAR *opt;
			};
			PSizeOpt psop[] = {
				{_T("A3")		,_T("(A3J|a3paper)")	},
				{_T("A4")		,_T("(A4J|a4paper)")	},
				{_T("A5")		,_T("(A5J|a5paper)")	},
				{_T("A6")		,_T("(A6J|a6paper)")	},
				{_T("A7")		,_T("(A7J|a7paper)")	},
				{_T("E3")		,_T("B3E")				},
				{_T("E4")		,_T("B4E")				},
				{_T("E5")		,_T("B5E")				},
				{_T("E6")		,_T("B6E")				},
				{_T("B3")		,_T("(B3J|b3paper)")	},
				{_T("B4")		,_T("(B4J|b4paper)")	},
				{_T("B5")		,_T("(B5J|b5paper)")	},
				{_T("B6")		,_T("(B6J|b6paper)")	},
				{_T("Letter")	,_T("letterpaper")		},
				{_T("Legal")	,_T("legalpaper")		},
				{_T("Executive"),_T("executivepaper")	},
				NULL
			};
#ifdef UNICODE
			std::wregex reg;
#else
			std::regex reg;
#endif
			for (PSizeOpt *p = psop; p->psize; ++p) {
				reg.assign(p->opt, std::regex::icase);
				if (std::regex_match(m_documentoption, reg)) {
					papersize = p->psize;
					break;
				}
			}
			break;
		}

		case DVIOUTSet::list:
			papersize = g_setting.DVIOUT_PaperSizeFromList();
			break;

		case DVIOUTSet::number:
			papersize = _T("F") + g_setting.DVIOUT_PaperSizeX() + _T(":") + g_setting.DVIOUT_PaperSizeY();
			break;
		}
	}

	ablib::system::DdeClient dde;
	if (!dde.Init(_T("dviout"), _T("dviout"))) {
		ablib::string cmd = _T("\"") + g_setting.DVIOUT_dvioutPath() + _T("\"");
		if (papersize != _T(""))cmd += _T(" -y:") + papersize;
		if (!Exec(cmd)) {
			m_mainwnd->GetWindow()->MsgBox(cmd + _T("̎sɎsD"), _T("TeXShell2"));
			return false;
		}
		if (!dde.Init(_T("dviout"), _T("dviout"))) {
			m_mainwnd->GetWindow()->MsgBox(_T("DDEɎsD"), _T("TeXShell"));
			return false;
		}
	}
	if (g_setting.DVIOUT_Maximize()) {
		HWND dviout = ::FindWindowEx(NULL, NULL, _T("dviout"), NULL);
		if (dviout) {
			if (::IsIconic(dviout)) {
				::ShowWindow(dviout, SW_RESTORE);
			}
			if (!::IsZoomed(dviout)) {
				::ShowWindow(dviout, SW_MAXIMIZE);
			}
		}
	}
	dde.Execute(_T("[WinNormal]"));
	ablib::string par;
	if (papersize != _T(""))par = _T(" -y:") + papersize;
	if (par != _T(""))dde.Execute(_T("[ValueSet(") + par + _T(")]"));
	ablib::string file = m_parentbase + _T(".dvi");
	if (dde.Request(_T("dvifile")) != file)dde.Execute(_T("[FileOpen(") + m_parentdir + file + _T(")]"));
	dde.Terminate();
	const TCHAR *p = file.c_str();
	return true;
}

bool TeX::PDFPreview() {
	if (m_filename == _T("")) {
		m_mainwnd->GetWindow()->MsgBox(_T("t@Cǂݍ܂Ă܂D"), _T("TeXShell2"));
		return false;
	}
	if (m_parent == _T("")) {
		m_mainwnd->GetWindow()->MsgBox(_T("et@C̎擾ɎsD"), _T("TeXShell2"));
		return false;
	}
	ablib::string cmd = _T("\"") + g_setting.PDFPreview_Program() + _T("\" ") +
		TransStr(g_setting.PDFPreview_Arg());
	if (!Exec(cmd)) {
		m_mainwnd->GetWindow()->MsgBox(cmd + _T("̎sɎsD"), _T("TeXShell"));
		return false;
	} else return true;
}

bool TeX::PSPreview() {
	if (m_filename == _T("")) {
		m_mainwnd->GetWindow()->MsgBox(_T("t@Cǂݍ܂Ă܂D"), _T("TeXShell2"));
		return false;
	}
	if (m_parent == _T("")) {
		m_mainwnd->GetWindow()->MsgBox(_T("et@C̎擾ɎsD"), _T("TeXShell2"));
		return false;
	}
	ablib::string cmd = _T("\"") + g_setting.PSPreview_Program() + _T("\" ") +
		TransStr(g_setting.PSPreview_Arg());
	if (!Exec(cmd)) {
		m_mainwnd->GetWindow()->MsgBox(cmd + _T("̎sɎsD"), _T("TeXShell"));
		return false;
	} else return true;
}

bool TeX::Exec(const ablib::string &cmd) {
	TCHAR curdir[4096];
	::GetCurrentDirectory(4095, curdir);
	PROCESS_INFORMATION pi;
	::ZeroMemory(&pi, sizeof(PROCESS_INFORMATION));
	STARTUPINFO si;
	si.cb = sizeof(STARTUPINFO);
	::GetStartupInfo(&si);
	if (!::CreateProcess(NULL, const_cast<TCHAR *>(cmd.c_str()), NULL, NULL, TRUE, 0, NULL, curdir, &si, &pi))return false;
	::WaitForInputIdle(pi.hProcess, INFINITE);
	::CloseHandle(pi.hProcess);
	::CloseHandle(pi.hThread);
	return true;
}

ablib::string TeX::TransStr(const ablib::string &s) {
	ablib::string rv;
	rv.reserve(s.length() + 40);
	ablib::string::size_type j = 0;
	ablib::string::size_type i;
	for (i = 0; i < s.length(); ++i) {
		if (s[i] == '%') {
			++i;
			if (s[i] == '%') {
				rv.append(s, j, i - j);
				j = i + 1;
				continue;
			} else {
				rv.append(s, j, i - j - 1);
				if (s[i] == 'f')rv += m_file;
				else if (s[i] == 'd')rv += m_dir;
				else if (s[i] == 'b')rv += m_base;
				else if (s[i] == 'k')rv += m_type;
				else if (s[i] == 'F')rv += m_parent;
				else if (s[i] == 'D')rv += m_parentdir;
				else if (s[i] == 'B')rv += m_parentbase;
				else if (s[i] == 'K')rv += m_parenttype;
				else rv += s[i];
				j = i + 1;
			}
		}
#ifndef UNICODE
		else if (ablib::IsKanji(s[i])) { ++i; continue; }
#endif
	}
	rv.append(s, j, i - j);
	return rv;
}
