#include "stdafx.h"
#include "tex2dviset.h"
#include "MakeIndexSet.h"
#include "BibTeXSet.h"
#include "Setting.h"
#include "ablib/lexical_cast.h"
#include "system/FindFile.h"
#include <system/Process.h>
#include <filesystem>
#include <boost/algorithm/string.hpp>

extern Setting g_setting;
extern HINSTANCE g_appint;

ablib::string TeX2DVISet::Which(const TCHAR *str){
	ablib::string env = getenv(_T("PATH"));
	ablib::string::size_type p = 0,q = 0;
	while((p = env.find(_T(";"),q)) != ablib::string::npos){
		ablib::string f = env.substr(q,p - q);
		if(f != _T("") && f[f.length() - 1] != '\\')f += _T("\\");
		if(::GetFileAttributes((f + str).c_str()) != -1)return f;
		q = p + 1;
	}
	return _T("");
}



bool TeX2DVISet::OnInitDialog(ablib::window::Dialog *wnd){
	if(g_setting.TeX2DVI_IsUsingFormat()){
		ablib::window::CheckBox(wnd,IDC_BYFORMAT).SetCheck(true);
		ablib::window::Edit(wnd,IDC_PROGRAM).SetReadOnly(true);
	}else{
		ablib::window::CheckBox(wnd,IDC_EDITPROGRAM).SetCheck(true);
		ablib::window::Combo(wnd,IDC_FORMAT).Enabled(false);
	}
	ablib::window::Edit(wnd,IDC_OPTION).SetText(g_setting.TeX2DVI_Option());
	if(g_setting.TeX2DVI_IsAnalyze())ablib::window::CheckBox(wnd,IDC_RECOMPILE).SetCheck(true);
	else ablib::window::Edit(wnd,IDC_RECOMPILENUM).SetReadOnly(true);
	ablib::window::Edit(wnd,IDC_RECOMPILENUM).SetText(ablib::lexical_cast<ablib::string>(g_setting.TeX2DVI_MaxCompile()));

	ablib::window::Combo combo(wnd,IDC_FORMAT);
	SetFormatList(combo);

	ablib::string format = g_setting.TeX2DVI_FormatFile();
	int n = combo.GetCount();
	for(int i = 0 ; i < n ; ++i){
		if(combo.GetText(i) == format){
			combo.SetCurSel(i);
			break;
		}
	}
	return true;
}

void TeX2DVISet::SetFormatList(ablib::window::Combo &combo) {
	ablib::system::Process process;
	ablib::system::Process::ProcessStartInfo si;
	si.Arguments = _T("kpsewhich -expand-path=$TEXFORMATS");
	si.RedirectStandardOutput = true;
	si.RedirectStandardError = true;
	si.CreateNoWindow = true;
	process.StartInfo(si);
	if (process.Start()) {
		std::string files;
		while (true) {
			TCHAR buf[4096];
			DWORD size = process.Read(buf, 4095);
			if (size == 0) {
				if (process.HasExited())break;
			} else {
				buf[size] = '\0';
				files += buf;
			}
		}
		std::list<ablib::string> filelist;
		boost::split(filelist, files, boost::is_any_of(_T(";")));
		std::vector<ablib::string> formatlist;
		for (auto dir : filelist) {
			dir.replace(_T("\n"), _T(""));
			dir.replace(_T("\r"), _T(""));
			dir.replace(_T("/"), _T("\\"));
			if (dir.length() == 0 || dir[dir.length() - 1] != '\\')dir += _T("\\");
			GetFormatList(dir, formatlist);
		}
		for (auto f : formatlist) {
			combo.AddString(std::tr2::sys::path(f.c_str()).filename().string());
		}
	} else OUTPUTLASTERROR(ablib::system::GetLastError());
}

void TeX2DVISet::GetFormatList(const ablib::string &dir, std::vector<ablib::string> &list) {
	ablib::system::FindFile ff;
	WIN32_FIND_DATA wfd;
	if (!ff.Begin(dir + _T("*.*")))return;
	while (ff.Next(&wfd)) {
		if (::GetFileAttributes((dir + wfd.cFileName).c_str()) & FILE_ATTRIBUTE_DIRECTORY) {
			GetFormatList(dir + wfd.cFileName + _T("\\"), list);
		} else {
			ablib::string s(std::tr2::sys::path(wfd.cFileName).extension().string());
			s.tolower();
			if (s == ".fmt") {
				list.push_back(dir + wfd.cFileName);
			}
		}
	}
}

bool TeX2DVISet::OnCommand(ablib::window::Dialog *wnd,int id,int param){
	switch(id){
	case IDC_BYFORMAT:
	case IDC_EDITPROGRAM:
		ablib::window::Combo(wnd,IDC_FORMAT).Enabled(
			ablib::window::CheckBox(wnd,IDC_BYFORMAT).GetCheck());
		ablib::window::Edit(wnd,IDC_PROGRAM).SetReadOnly(
			!ablib::window::CheckBox(wnd,IDC_EDITPROGRAM).GetCheck());
		return true;
		
	case IDC_RECOMPILE:
		ablib::window::Edit(wnd,IDC_RECOMPILENUM).SetReadOnly(
			!ablib::window::CheckBox(wnd,IDC_RECOMPILE).GetCheck());
		return true;

	case IDOK:
		g_setting.WriteTeX2DVI_IsUsingFormat(ablib::window::CheckBox(wnd,IDC_BYFORMAT).GetCheck());
		g_setting.WriteTeX2DVI_FormatFile(ablib::window::Combo(wnd,IDC_FORMAT).GetText());
		g_setting.WriteTeX2DVI_Program(ablib::window::Edit(wnd,IDC_PROGRAM).GetText());
		g_setting.WriteTeX2DVI_Option(ablib::window::Edit(wnd,IDC_OPTION).GetText());
		g_setting.WriteTeX2DVI_IsAnalyze(ablib::window::CheckBox(wnd,IDC_RECOMPILE).GetCheck());
		g_setting.WriteTeX2DVI_MaxCompile(
			ablib::lexical_cast<int>(ablib::window::Edit(wnd,IDC_RECOMPILENUM).GetText()));
		wnd->Destroy(TRUE);
		return true;
	
	case IDCANCEL:
		wnd->Destroy(FALSE);
		return true;
		
	case IDC_MAKEINDEX:
	{
		MakeIndexSet mis(g_appint);
		mis.Create(wnd);
		return true;
	}

	case IDC_BIBTEX:
	{
		BibTeXSet bts(g_appint);
		bts.Create(wnd);
		return true;
	}
	default:
		return false;
	}
}



